import { resolve, join } from 'path';
import { existsSync } from 'fs';
import defu from 'defu';

var name = "@nuxtjs/laravel-echo";
var version = "2.0.0-alpha.3";

const DEFAULTS = {
  broadcaster: "null",
  encrypted: false,
  authModule: false,
  connectOnLogin: true,
  disconnectOnLogout: true
};
const nuxtModule = function(moduleOptions) {
  this.nuxt.hook("builder:extendPlugins", (plugins) => {
    const options = defu(this.options.echo || {}, moduleOptions, DEFAULTS);
    const runtimeDir = resolve(__dirname, "runtime");
    this.options.alias["~echo"] = runtimeDir;
    this.options.build.transpile.push(runtimeDir, "laravel-echo", "defu");
    const optionsPath = this.nuxt.resolver.resolveAlias(options.optionsPath || join(this.options.dir.app || "app", "laravel-echo", "options.js"));
    delete options.optionsPath;
    this.addTemplate({
      fileName: `laravel-echo/options.${optionsPath && optionsPath.endsWith("ts") ? "ts" : "js"}`,
      src: existsSync(optionsPath) ? optionsPath : resolve(__dirname, "./runtime/options.js"),
      options
    });
    const {dst} = this.addTemplate({
      src: resolve(__dirname, "./runtime/plugin.js"),
      fileName: "laravel-echo/plugin.js",
      options
    });
    plugins.push({
      src: resolve(this.options.buildDir, dst),
      ssr: false
    });
    if (options.plugins) {
      options.plugins.forEach((p) => plugins.push({src: p, ssr: false}));
      delete options.plugins;
    }
  });
};
nuxtModule.meta = {name, version};

export default nuxtModule;
